# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *


class DeviceSettingsMessage(DataMessage):
    ID = 2
    NAME = 'device_settings'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=DeviceSettingsMessage.NAME,
                         global_id=DeviceSettingsMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             DeviceSettingsActiveTimeZoneField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsActiveTimeZoneField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsUtcOffsetField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsUtcOffsetField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsTimeOffsetField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsTimeOffsetField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsTimeModeField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsTimeModeField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsTimeZoneOffsetField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsTimeZoneOffsetField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsBacklightModeField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsBacklightModeField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsActivityTrackerEnabledField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsActivityTrackerEnabledField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsClockTimeField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsClockTimeField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsPagesEnabledField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsPagesEnabledField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsMoveAlertEnabledField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsMoveAlertEnabledField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsDateModeField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsDateModeField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsDisplayOrientationField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsDisplayOrientationField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsMountingSideField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsMountingSideField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsDefaultPageField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsDefaultPageField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsAutosyncMinStepsField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsAutosyncMinStepsField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsAutosyncMinTimeField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsAutosyncMinTimeField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsLactateThresholdAutodetectEnabledField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsLactateThresholdAutodetectEnabledField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsBleAutoUploadEnabledField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsBleAutoUploadEnabledField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsAutoSyncFrequencyField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsAutoSyncFrequencyField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsAutoActivityDetectField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsAutoActivityDetectField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsNumberOfScreensField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsNumberOfScreensField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsSmartNotificationDisplayOrientationField(
                                 size=self.__get_field_size(definition_message,
                                                            DeviceSettingsSmartNotificationDisplayOrientationField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsTapInterfaceField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsTapInterfaceField.ID),
                                 growable=definition_message is None),
                             DeviceSettingsTapSensitivityField(
                                 size=self.__get_field_size(definition_message, DeviceSettingsTapSensitivityField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def active_time_zone(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsActiveTimeZoneField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @active_time_zone.setter
    def active_time_zone(self, value: int):
        field = self.get_field(DeviceSettingsActiveTimeZoneField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def utc_offset(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsUtcOffsetField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @utc_offset.setter
    def utc_offset(self, value: int):
        field = self.get_field(DeviceSettingsUtcOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def time_offset(self) -> Optional[list[int]]:
        field = self.get_field(DeviceSettingsTimeOffsetField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @time_offset.setter
    def time_offset(self, value: list[int]):
        field = self.get_field(DeviceSettingsTimeOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def time_mode(self) -> Optional[list[TimeMode]]:
        field = self.get_field(DeviceSettingsTimeModeField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @time_mode.setter
    def time_mode(self, value: list[TimeMode]):
        field = self.get_field(DeviceSettingsTimeModeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def time_zone_offset(self) -> Optional[list[float]]:
        field = self.get_field(DeviceSettingsTimeZoneOffsetField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @time_zone_offset.setter
    def time_zone_offset(self, value: list[float]):
        field = self.get_field(DeviceSettingsTimeZoneOffsetField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def backlight_mode(self) -> Optional[BacklightMode]:
        field = self.get_field(DeviceSettingsBacklightModeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @backlight_mode.setter
    def backlight_mode(self, value: BacklightMode):
        field = self.get_field(DeviceSettingsBacklightModeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def activity_tracker_enabled(self) -> Optional[bool]:
        field = self.get_field(DeviceSettingsActivityTrackerEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @activity_tracker_enabled.setter
    def activity_tracker_enabled(self, value: bool):
        field = self.get_field(DeviceSettingsActivityTrackerEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def clock_time(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsClockTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @clock_time.setter
    def clock_time(self, value: int):
        field = self.get_field(DeviceSettingsClockTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def pages_enabled(self) -> Optional[list[int]]:
        field = self.get_field(DeviceSettingsPagesEnabledField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @pages_enabled.setter
    def pages_enabled(self, value: list[int]):
        field = self.get_field(DeviceSettingsPagesEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def move_alert_enabled(self) -> Optional[bool]:
        field = self.get_field(DeviceSettingsMoveAlertEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @move_alert_enabled.setter
    def move_alert_enabled(self, value: bool):
        field = self.get_field(DeviceSettingsMoveAlertEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def date_mode(self) -> Optional[DateMode]:
        field = self.get_field(DeviceSettingsDateModeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @date_mode.setter
    def date_mode(self, value: DateMode):
        field = self.get_field(DeviceSettingsDateModeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def display_orientation(self) -> Optional[DisplayOrientation]:
        field = self.get_field(DeviceSettingsDisplayOrientationField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @display_orientation.setter
    def display_orientation(self, value: DisplayOrientation):
        field = self.get_field(DeviceSettingsDisplayOrientationField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def mounting_side(self) -> Optional[Side]:
        field = self.get_field(DeviceSettingsMountingSideField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @mounting_side.setter
    def mounting_side(self, value: Side):
        field = self.get_field(DeviceSettingsMountingSideField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def default_page(self) -> Optional[list[int]]:
        field = self.get_field(DeviceSettingsDefaultPageField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @default_page.setter
    def default_page(self, value: list[int]):
        field = self.get_field(DeviceSettingsDefaultPageField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def autosync_min_steps(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsAutosyncMinStepsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @autosync_min_steps.setter
    def autosync_min_steps(self, value: int):
        field = self.get_field(DeviceSettingsAutosyncMinStepsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def autosync_min_time(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsAutosyncMinTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @autosync_min_time.setter
    def autosync_min_time(self, value: int):
        field = self.get_field(DeviceSettingsAutosyncMinTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def lactate_threshold_autodetect_enabled(self) -> Optional[bool]:
        field = self.get_field(DeviceSettingsLactateThresholdAutodetectEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @lactate_threshold_autodetect_enabled.setter
    def lactate_threshold_autodetect_enabled(self, value: bool):
        field = self.get_field(DeviceSettingsLactateThresholdAutodetectEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def ble_auto_upload_enabled(self) -> Optional[bool]:
        field = self.get_field(DeviceSettingsBleAutoUploadEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @ble_auto_upload_enabled.setter
    def ble_auto_upload_enabled(self, value: bool):
        field = self.get_field(DeviceSettingsBleAutoUploadEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def auto_sync_frequency(self) -> Optional[AutoSyncFrequency]:
        field = self.get_field(DeviceSettingsAutoSyncFrequencyField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @auto_sync_frequency.setter
    def auto_sync_frequency(self, value: AutoSyncFrequency):
        field = self.get_field(DeviceSettingsAutoSyncFrequencyField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def auto_activity_detect(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsAutoActivityDetectField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @auto_activity_detect.setter
    def auto_activity_detect(self, value: int):
        field = self.get_field(DeviceSettingsAutoActivityDetectField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def number_of_screens(self) -> Optional[int]:
        field = self.get_field(DeviceSettingsNumberOfScreensField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @number_of_screens.setter
    def number_of_screens(self, value: int):
        field = self.get_field(DeviceSettingsNumberOfScreensField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def smart_notification_display_orientation(self) -> Optional[DisplayOrientation]:
        field = self.get_field(DeviceSettingsSmartNotificationDisplayOrientationField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @smart_notification_display_orientation.setter
    def smart_notification_display_orientation(self, value: DisplayOrientation):
        field = self.get_field(DeviceSettingsSmartNotificationDisplayOrientationField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def tap_interface(self) -> Optional[SwitchType]:
        field = self.get_field(DeviceSettingsTapInterfaceField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @tap_interface.setter
    def tap_interface(self, value: SwitchType):
        field = self.get_field(DeviceSettingsTapInterfaceField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def tap_sensitivity(self) -> Optional[TapSensitivity]:
        field = self.get_field(DeviceSettingsTapSensitivityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @tap_sensitivity.setter
    def tap_sensitivity(self, value: TapSensitivity):
        field = self.get_field(DeviceSettingsTapSensitivityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class DeviceSettingsActiveTimeZoneField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='active_time_zone',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsUtcOffsetField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='utc_offset',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsTimeOffsetField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='time_offset',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsTimeModeField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='time_mode',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsTimeZoneOffsetField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='time_zone_offset',
            field_id=self.ID,
            base_type=BaseType.SINT8,
            offset=0,
            scale=4,
            size=size,
            units='hr',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsBacklightModeField(Field):
    ID = 12

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='backlight_mode',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsActivityTrackerEnabledField(Field):
    ID = 36

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='activity_tracker_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsClockTimeField(Field):
    ID = 39

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='clock_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsPagesEnabledField(Field):
    ID = 40

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='pages_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsMoveAlertEnabledField(Field):
    ID = 46

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='move_alert_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsDateModeField(Field):
    ID = 47

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='date_mode',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsDisplayOrientationField(Field):
    ID = 55

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='display_orientation',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsMountingSideField(Field):
    ID = 56

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='mounting_side',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsDefaultPageField(Field):
    ID = 57

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='default_page',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsAutosyncMinStepsField(Field):
    ID = 58

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='autosync_min_steps',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='steps',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsAutosyncMinTimeField(Field):
    ID = 59

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='autosync_min_time',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='minutes',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsLactateThresholdAutodetectEnabledField(Field):
    ID = 80

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='lactate_threshold_autodetect_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsBleAutoUploadEnabledField(Field):
    ID = 86

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='ble_auto_upload_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsAutoSyncFrequencyField(Field):
    ID = 89

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='auto_sync_frequency',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsAutoActivityDetectField(Field):
    ID = 90

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='auto_activity_detect',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsNumberOfScreensField(Field):
    ID = 94

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='number_of_screens',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsSmartNotificationDisplayOrientationField(Field):
    ID = 95

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='smart_notification_display_orientation',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsTapInterfaceField(Field):
    ID = 134

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='tap_interface',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DeviceSettingsTapSensitivityField(Field):
    ID = 174

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='tap_sensitivity',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=1,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list

from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.generic_message import GenericMessage
from fit_tool.profile.messages.accelerometer_data_message import AccelerometerDataMessage
from fit_tool.profile.messages.activity_message import ActivityMessage
from fit_tool.profile.messages.ant_channel_id_message import AntChannelIdMessage
from fit_tool.profile.messages.ant_rx_message import AntRxMessage
from fit_tool.profile.messages.ant_tx_message import AntTxMessage
from fit_tool.profile.messages.aviation_attitude_message import AviationAttitudeMessage
from fit_tool.profile.messages.barometer_data_message import BarometerDataMessage
from fit_tool.profile.messages.bike_profile_message import BikeProfileMessage
from fit_tool.profile.messages.blood_pressure_message import BloodPressureMessage
from fit_tool.profile.messages.cadence_zone_message import CadenceZoneMessage
from fit_tool.profile.messages.camera_event_message import CameraEventMessage
from fit_tool.profile.messages.capabilities_message import CapabilitiesMessage
from fit_tool.profile.messages.climb_pro_message import ClimbProMessage
from fit_tool.profile.messages.connectivity_message import ConnectivityMessage
from fit_tool.profile.messages.course_message import CourseMessage
from fit_tool.profile.messages.course_point_message import CoursePointMessage
from fit_tool.profile.messages.developer_data_id_message import DeveloperDataIdMessage
from fit_tool.profile.messages.device_info_message import DeviceInfoMessage
from fit_tool.profile.messages.device_settings_message import DeviceSettingsMessage
from fit_tool.profile.messages.dive_alarm_message import DiveAlarmMessage
from fit_tool.profile.messages.dive_gas_message import DiveGasMessage
from fit_tool.profile.messages.dive_settings_message import DiveSettingsMessage
from fit_tool.profile.messages.dive_summary_message import DiveSummaryMessage
from fit_tool.profile.messages.event_message import EventMessage
from fit_tool.profile.messages.exd_data_concept_configuration_message import ExdDataConceptConfigurationMessage
from fit_tool.profile.messages.exd_data_field_configuration_message import ExdDataFieldConfigurationMessage
from fit_tool.profile.messages.exd_screen_configuration_message import ExdScreenConfigurationMessage
from fit_tool.profile.messages.exercise_title_message import ExerciseTitleMessage
from fit_tool.profile.messages.field_capabilities_message import FieldCapabilitiesMessage
from fit_tool.profile.messages.field_description_message import FieldDescriptionMessage
from fit_tool.profile.messages.file_capabilities_message import FileCapabilitiesMessage
from fit_tool.profile.messages.file_creator_message import FileCreatorMessage
from fit_tool.profile.messages.file_id_message import FileIdMessage
from fit_tool.profile.messages.goal_message import GoalMessage
from fit_tool.profile.messages.gps_metadata_message import GpsMetadataMessage
from fit_tool.profile.messages.gyroscope_data_message import GyroscopeDataMessage
from fit_tool.profile.messages.hr_message import HrMessage
from fit_tool.profile.messages.hr_zone_message import HrZoneMessage
from fit_tool.profile.messages.hrm_profile_message import HrmProfileMessage
from fit_tool.profile.messages.hrv_message import HrvMessage
from fit_tool.profile.messages.jump_message import JumpMessage
from fit_tool.profile.messages.lap_message import LapMessage
from fit_tool.profile.messages.length_message import LengthMessage
from fit_tool.profile.messages.magnetometer_data_message import MagnetometerDataMessage
from fit_tool.profile.messages.memo_glob_message import MemoGlobMessage
from fit_tool.profile.messages.mesg_capabilities_message import MesgCapabilitiesMessage
from fit_tool.profile.messages.met_zone_message import MetZoneMessage
from fit_tool.profile.messages.monitoring_info_message import MonitoringInfoMessage
from fit_tool.profile.messages.monitoring_message import MonitoringMessage
from fit_tool.profile.messages.nmea_sentence_message import NmeaSentenceMessage
from fit_tool.profile.messages.obdii_data_message import ObdiiDataMessage
from fit_tool.profile.messages.ohr_settings_message import OhrSettingsMessage
from fit_tool.profile.messages.one_d_sensor_calibration_message import OneDSensorCalibrationMessage
from fit_tool.profile.messages.power_zone_message import PowerZoneMessage
from fit_tool.profile.messages.record_message import RecordMessage
from fit_tool.profile.messages.schedule_message import ScheduleMessage
from fit_tool.profile.messages.sdm_profile_message import SdmProfileMessage
from fit_tool.profile.messages.segment_file_message import SegmentFileMessage
from fit_tool.profile.messages.segment_id_message import SegmentIdMessage
from fit_tool.profile.messages.segment_lap_message import SegmentLapMessage
from fit_tool.profile.messages.segment_leaderboard_entry_message import SegmentLeaderboardEntryMessage
from fit_tool.profile.messages.segment_point_message import SegmentPointMessage
from fit_tool.profile.messages.session_message import SessionMessage
from fit_tool.profile.messages.set_message import SetMessage
from fit_tool.profile.messages.slave_device_message import SlaveDeviceMessage
from fit_tool.profile.messages.software_message import SoftwareMessage
from fit_tool.profile.messages.speed_zone_message import SpeedZoneMessage
from fit_tool.profile.messages.sport_message import SportMessage
from fit_tool.profile.messages.stress_level_message import StressLevelMessage
from fit_tool.profile.messages.three_d_sensor_calibration_message import ThreeDSensorCalibrationMessage
from fit_tool.profile.messages.timestamp_correlation_message import TimestampCorrelationMessage
from fit_tool.profile.messages.totals_message import TotalsMessage
from fit_tool.profile.messages.training_file_message import TrainingFileMessage
from fit_tool.profile.messages.user_profile_message import UserProfileMessage
from fit_tool.profile.messages.video_clip_message import VideoClipMessage
from fit_tool.profile.messages.video_description_message import VideoDescriptionMessage
from fit_tool.profile.messages.video_frame_message import VideoFrameMessage
from fit_tool.profile.messages.video_message import VideoMessage
from fit_tool.profile.messages.video_title_message import VideoTitleMessage
from fit_tool.profile.messages.watchface_settings_message import WatchfaceSettingsMessage
from fit_tool.profile.messages.weather_alert_message import WeatherAlertMessage
from fit_tool.profile.messages.weather_conditions_message import WeatherConditionsMessage
from fit_tool.profile.messages.weight_scale_message import WeightScaleMessage
from fit_tool.profile.messages.workout_message import WorkoutMessage
from fit_tool.profile.messages.workout_session_message import WorkoutSessionMessage
from fit_tool.profile.messages.workout_step_message import WorkoutStepMessage
from fit_tool.profile.messages.zones_target_message import ZonesTargetMessage


class MessageFactory:

    @staticmethod
    def from_definition(definition_message: DefinitionMessage, developer_fields: list[DeveloperField]) -> DataMessage:

        if definition_message.global_id == FileIdMessage.ID:
            return FileIdMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == FileCreatorMessage.ID:
            return FileCreatorMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == TimestampCorrelationMessage.ID:
            return TimestampCorrelationMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SoftwareMessage.ID:
            return SoftwareMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SlaveDeviceMessage.ID:
            return SlaveDeviceMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == CapabilitiesMessage.ID:
            return CapabilitiesMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == FileCapabilitiesMessage.ID:
            return FileCapabilitiesMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MesgCapabilitiesMessage.ID:
            return MesgCapabilitiesMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == FieldCapabilitiesMessage.ID:
            return FieldCapabilitiesMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DeviceSettingsMessage.ID:
            return DeviceSettingsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == UserProfileMessage.ID:
            return UserProfileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == HrmProfileMessage.ID:
            return HrmProfileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SdmProfileMessage.ID:
            return SdmProfileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == BikeProfileMessage.ID:
            return BikeProfileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ConnectivityMessage.ID:
            return ConnectivityMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WatchfaceSettingsMessage.ID:
            return WatchfaceSettingsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == OhrSettingsMessage.ID:
            return OhrSettingsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ZonesTargetMessage.ID:
            return ZonesTargetMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SportMessage.ID:
            return SportMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == HrZoneMessage.ID:
            return HrZoneMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SpeedZoneMessage.ID:
            return SpeedZoneMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == CadenceZoneMessage.ID:
            return CadenceZoneMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == PowerZoneMessage.ID:
            return PowerZoneMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MetZoneMessage.ID:
            return MetZoneMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DiveSettingsMessage.ID:
            return DiveSettingsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DiveAlarmMessage.ID:
            return DiveAlarmMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DiveGasMessage.ID:
            return DiveGasMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == GoalMessage.ID:
            return GoalMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ActivityMessage.ID:
            return ActivityMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SessionMessage.ID:
            return SessionMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == LapMessage.ID:
            return LapMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == LengthMessage.ID:
            return LengthMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == RecordMessage.ID:
            return RecordMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == EventMessage.ID:
            return EventMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DeviceInfoMessage.ID:
            return DeviceInfoMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == TrainingFileMessage.ID:
            return TrainingFileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == HrvMessage.ID:
            return HrvMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WeatherConditionsMessage.ID:
            return WeatherConditionsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WeatherAlertMessage.ID:
            return WeatherAlertMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == GpsMetadataMessage.ID:
            return GpsMetadataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == CameraEventMessage.ID:
            return CameraEventMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == GyroscopeDataMessage.ID:
            return GyroscopeDataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == AccelerometerDataMessage.ID:
            return AccelerometerDataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MagnetometerDataMessage.ID:
            return MagnetometerDataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == BarometerDataMessage.ID:
            return BarometerDataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ThreeDSensorCalibrationMessage.ID:
            return ThreeDSensorCalibrationMessage(definition_message=definition_message,
                                                  developer_fields=developer_fields)

        elif definition_message.global_id == OneDSensorCalibrationMessage.ID:
            return OneDSensorCalibrationMessage(definition_message=definition_message,
                                                developer_fields=developer_fields)

        elif definition_message.global_id == VideoFrameMessage.ID:
            return VideoFrameMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ObdiiDataMessage.ID:
            return ObdiiDataMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == NmeaSentenceMessage.ID:
            return NmeaSentenceMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == AviationAttitudeMessage.ID:
            return AviationAttitudeMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == VideoMessage.ID:
            return VideoMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == VideoTitleMessage.ID:
            return VideoTitleMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == VideoDescriptionMessage.ID:
            return VideoDescriptionMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == VideoClipMessage.ID:
            return VideoClipMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SetMessage.ID:
            return SetMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == JumpMessage.ID:
            return JumpMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == CourseMessage.ID:
            return CourseMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == CoursePointMessage.ID:
            return CoursePointMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SegmentIdMessage.ID:
            return SegmentIdMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SegmentLeaderboardEntryMessage.ID:
            return SegmentLeaderboardEntryMessage(definition_message=definition_message,
                                                  developer_fields=developer_fields)

        elif definition_message.global_id == SegmentPointMessage.ID:
            return SegmentPointMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SegmentLapMessage.ID:
            return SegmentLapMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == SegmentFileMessage.ID:
            return SegmentFileMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WorkoutMessage.ID:
            return WorkoutMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WorkoutSessionMessage.ID:
            return WorkoutSessionMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WorkoutStepMessage.ID:
            return WorkoutStepMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ExerciseTitleMessage.ID:
            return ExerciseTitleMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ScheduleMessage.ID:
            return ScheduleMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == TotalsMessage.ID:
            return TotalsMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == WeightScaleMessage.ID:
            return WeightScaleMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == BloodPressureMessage.ID:
            return BloodPressureMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MonitoringInfoMessage.ID:
            return MonitoringInfoMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MonitoringMessage.ID:
            return MonitoringMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == HrMessage.ID:
            return HrMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == StressLevelMessage.ID:
            return StressLevelMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == MemoGlobMessage.ID:
            return MemoGlobMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == AntChannelIdMessage.ID:
            return AntChannelIdMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == AntRxMessage.ID:
            return AntRxMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == AntTxMessage.ID:
            return AntTxMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ExdScreenConfigurationMessage.ID:
            return ExdScreenConfigurationMessage(definition_message=definition_message,
                                                 developer_fields=developer_fields)

        elif definition_message.global_id == ExdDataFieldConfigurationMessage.ID:
            return ExdDataFieldConfigurationMessage(definition_message=definition_message,
                                                    developer_fields=developer_fields)

        elif definition_message.global_id == ExdDataConceptConfigurationMessage.ID:
            return ExdDataConceptConfigurationMessage(definition_message=definition_message,
                                                      developer_fields=developer_fields)

        elif definition_message.global_id == FieldDescriptionMessage.ID:
            return FieldDescriptionMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DeveloperDataIdMessage.ID:
            return DeveloperDataIdMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == DiveSummaryMessage.ID:
            return DiveSummaryMessage(definition_message=definition_message, developer_fields=developer_fields)

        elif definition_message.global_id == ClimbProMessage.ID:
            return ClimbProMessage(definition_message=definition_message, developer_fields=developer_fields)
        else:
            return GenericMessage(definition_message=definition_message, developer_fields=developer_fields)



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class DiveSettingsMessage(DataMessage):
    ID = 258
    NAME = 'dive_settings'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=DiveSettingsMessage.NAME,
                         global_id=DiveSettingsMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             MessageIndexField(
                                 size=self.__get_field_size(definition_message, MessageIndexField.ID),
                                 growable=definition_message is None),
                             DiveSettingsNameField(
                                 size=self.__get_field_size(definition_message, DiveSettingsNameField.ID),
                                 growable=definition_message is None),
                             DiveSettingsModelField(
                                 size=self.__get_field_size(definition_message, DiveSettingsModelField.ID),
                                 growable=definition_message is None),
                             DiveSettingsGfLowField(
                                 size=self.__get_field_size(definition_message, DiveSettingsGfLowField.ID),
                                 growable=definition_message is None),
                             DiveSettingsGfHighField(
                                 size=self.__get_field_size(definition_message, DiveSettingsGfHighField.ID),
                                 growable=definition_message is None),
                             DiveSettingsWaterTypeField(
                                 size=self.__get_field_size(definition_message, DiveSettingsWaterTypeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsWaterDensityField(
                                 size=self.__get_field_size(definition_message, DiveSettingsWaterDensityField.ID),
                                 growable=definition_message is None),
                             DiveSettingsPo2WarnField(
                                 size=self.__get_field_size(definition_message, DiveSettingsPo2WarnField.ID),
                                 growable=definition_message is None),
                             DiveSettingsPo2CriticalField(
                                 size=self.__get_field_size(definition_message, DiveSettingsPo2CriticalField.ID),
                                 growable=definition_message is None),
                             DiveSettingsPo2DecoField(
                                 size=self.__get_field_size(definition_message, DiveSettingsPo2DecoField.ID),
                                 growable=definition_message is None),
                             DiveSettingsSafetyStopEnabledField(
                                 size=self.__get_field_size(definition_message, DiveSettingsSafetyStopEnabledField.ID),
                                 growable=definition_message is None),
                             DiveSettingsBottomDepthField(
                                 size=self.__get_field_size(definition_message, DiveSettingsBottomDepthField.ID),
                                 growable=definition_message is None),
                             DiveSettingsBottomTimeField(
                                 size=self.__get_field_size(definition_message, DiveSettingsBottomTimeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsApneaCountdownEnabledField(
                                 size=self.__get_field_size(definition_message,
                                                            DiveSettingsApneaCountdownEnabledField.ID),
                                 growable=definition_message is None),
                             DiveSettingsApneaCountdownTimeField(
                                 size=self.__get_field_size(definition_message, DiveSettingsApneaCountdownTimeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsBacklightModeField(
                                 size=self.__get_field_size(definition_message, DiveSettingsBacklightModeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsBacklightBrightnessField(
                                 size=self.__get_field_size(definition_message,
                                                            DiveSettingsBacklightBrightnessField.ID),
                                 growable=definition_message is None),
                             DiveSettingsBacklightTimeoutField(
                                 size=self.__get_field_size(definition_message, DiveSettingsBacklightTimeoutField.ID),
                                 growable=definition_message is None),
                             DiveSettingsRepeatDiveIntervalField(
                                 size=self.__get_field_size(definition_message, DiveSettingsRepeatDiveIntervalField.ID),
                                 growable=definition_message is None),
                             DiveSettingsSafetyStopTimeField(
                                 size=self.__get_field_size(definition_message, DiveSettingsSafetyStopTimeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsHeartRateSourceTypeField(
                                 size=self.__get_field_size(definition_message,
                                                            DiveSettingsHeartRateSourceTypeField.ID),
                                 growable=definition_message is None),
                             DiveSettingsHeartRateSourceField(
                                 size=self.__get_field_size(definition_message, DiveSettingsHeartRateSourceField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(MessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(MessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def dive_settings_name(self) -> Optional[str]:
        field = self.get_field(DiveSettingsNameField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @dive_settings_name.setter
    def dive_settings_name(self, value: str):
        field = self.get_field(DiveSettingsNameField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def model(self) -> Optional[TissueModelType]:
        field = self.get_field(DiveSettingsModelField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @model.setter
    def model(self, value: TissueModelType):
        field = self.get_field(DiveSettingsModelField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def gf_low(self) -> Optional[int]:
        field = self.get_field(DiveSettingsGfLowField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @gf_low.setter
    def gf_low(self, value: int):
        field = self.get_field(DiveSettingsGfLowField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def gf_high(self) -> Optional[int]:
        field = self.get_field(DiveSettingsGfHighField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @gf_high.setter
    def gf_high(self, value: int):
        field = self.get_field(DiveSettingsGfHighField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def water_type(self) -> Optional[WaterType]:
        field = self.get_field(DiveSettingsWaterTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @water_type.setter
    def water_type(self, value: WaterType):
        field = self.get_field(DiveSettingsWaterTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def water_density(self) -> Optional[float]:
        field = self.get_field(DiveSettingsWaterDensityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @water_density.setter
    def water_density(self, value: float):
        field = self.get_field(DiveSettingsWaterDensityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def po2_warn(self) -> Optional[float]:
        field = self.get_field(DiveSettingsPo2WarnField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @po2_warn.setter
    def po2_warn(self, value: float):
        field = self.get_field(DiveSettingsPo2WarnField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def po2_critical(self) -> Optional[float]:
        field = self.get_field(DiveSettingsPo2CriticalField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @po2_critical.setter
    def po2_critical(self, value: float):
        field = self.get_field(DiveSettingsPo2CriticalField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def po2_deco(self) -> Optional[float]:
        field = self.get_field(DiveSettingsPo2DecoField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @po2_deco.setter
    def po2_deco(self, value: float):
        field = self.get_field(DiveSettingsPo2DecoField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def safety_stop_enabled(self) -> Optional[bool]:
        field = self.get_field(DiveSettingsSafetyStopEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @safety_stop_enabled.setter
    def safety_stop_enabled(self, value: bool):
        field = self.get_field(DiveSettingsSafetyStopEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def bottom_depth(self) -> Optional[float]:
        field = self.get_field(DiveSettingsBottomDepthField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @bottom_depth.setter
    def bottom_depth(self, value: float):
        field = self.get_field(DiveSettingsBottomDepthField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def bottom_time(self) -> Optional[int]:
        field = self.get_field(DiveSettingsBottomTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @bottom_time.setter
    def bottom_time(self, value: int):
        field = self.get_field(DiveSettingsBottomTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def apnea_countdown_enabled(self) -> Optional[bool]:
        field = self.get_field(DiveSettingsApneaCountdownEnabledField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @apnea_countdown_enabled.setter
    def apnea_countdown_enabled(self, value: bool):
        field = self.get_field(DiveSettingsApneaCountdownEnabledField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def apnea_countdown_time(self) -> Optional[int]:
        field = self.get_field(DiveSettingsApneaCountdownTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @apnea_countdown_time.setter
    def apnea_countdown_time(self, value: int):
        field = self.get_field(DiveSettingsApneaCountdownTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def backlight_mode(self) -> Optional[DiveBacklightMode]:
        field = self.get_field(DiveSettingsBacklightModeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @backlight_mode.setter
    def backlight_mode(self, value: DiveBacklightMode):
        field = self.get_field(DiveSettingsBacklightModeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def backlight_brightness(self) -> Optional[int]:
        field = self.get_field(DiveSettingsBacklightBrightnessField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @backlight_brightness.setter
    def backlight_brightness(self, value: int):
        field = self.get_field(DiveSettingsBacklightBrightnessField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def backlight_timeout(self) -> Optional[int]:
        field = self.get_field(DiveSettingsBacklightTimeoutField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @backlight_timeout.setter
    def backlight_timeout(self, value: int):
        field = self.get_field(DiveSettingsBacklightTimeoutField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def repeat_dive_interval(self) -> Optional[int]:
        field = self.get_field(DiveSettingsRepeatDiveIntervalField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @repeat_dive_interval.setter
    def repeat_dive_interval(self, value: int):
        field = self.get_field(DiveSettingsRepeatDiveIntervalField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def safety_stop_time(self) -> Optional[int]:
        field = self.get_field(DiveSettingsSafetyStopTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @safety_stop_time.setter
    def safety_stop_time(self, value: int):
        field = self.get_field(DiveSettingsSafetyStopTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def heart_rate_source_type(self) -> Optional[SourceType]:
        field = self.get_field(DiveSettingsHeartRateSourceTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @heart_rate_source_type.setter
    def heart_rate_source_type(self, value: SourceType):
        field = self.get_field(DiveSettingsHeartRateSourceTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def heart_rate_source(self) -> Optional[int]:
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @heart_rate_source.setter
    def heart_rate_source(self, value: int):
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def heart_rate_antplus_device_type(self) -> Optional[int]:
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)
        type_field = self.get_field(DiveSettingsHeartRateSourceTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @heart_rate_antplus_device_type.setter
    def heart_rate_antplus_device_type(self, value: int):
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def heart_rate_local_device_type(self) -> Optional[int]:
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)
        type_field = self.get_field(DiveSettingsHeartRateSourceTypeField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [5]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @heart_rate_local_device_type.setter
    def heart_rate_local_device_type(self, value: int):
        field = self.get_field(DiveSettingsHeartRateSourceField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class MessageIndexField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsNameField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='name',
            field_id=self.ID,
            base_type=BaseType.STRING,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsModelField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='model',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsGfLowField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='gf_low',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsGfHighField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='gf_high',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsWaterTypeField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='water_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsWaterDensityField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='water_density',
            field_id=self.ID,
            base_type=BaseType.FLOAT32,
            offset=0,
            scale=1,
            size=size,
            units='kg/m^3',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsPo2WarnField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='po2_warn',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=100,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsPo2CriticalField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='po2_critical',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=100,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsPo2DecoField(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='po2_deco',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=100,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsSafetyStopEnabledField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='safety_stop_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsBottomDepthField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='bottom_depth',
            field_id=self.ID,
            base_type=BaseType.FLOAT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsBottomTimeField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='bottom_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsApneaCountdownEnabledField(Field):
    ID = 12

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='apnea_countdown_enabled',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsApneaCountdownTimeField(Field):
    ID = 13

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='apnea_countdown_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsBacklightModeField(Field):
    ID = 14

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='backlight_mode',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsBacklightBrightnessField(Field):
    ID = 15

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='backlight_brightness',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsBacklightTimeoutField(Field):
    ID = 16

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='backlight_timeout',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsRepeatDiveIntervalField(Field):
    ID = 17

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='repeat_dive_interval',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsSafetyStopTimeField(Field):
    ID = 18

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='safety_stop_time',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsHeartRateSourceTypeField(Field):
    ID = 19

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='heart_rate_source_type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSettingsHeartRateSourceField(Field):
    ID = 20

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='heart_rate_source',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='heart_rate_antplus_device_type',
                    base_type=BaseType.UINT8,
                    scale=1,
                    offset=0,
                    reference_map={
                        DiveSettingsHeartRateSourceTypeField.ID: [1]
                    }),
                SubField(
                    name='heart_rate_local_device_type',
                    base_type=BaseType.UINT8,
                    scale=1,
                    offset=0,
                    reference_map={
                        DiveSettingsHeartRateSourceTypeField.ID: [5]
                    })
            ]
        )



import logging

# noinspection SpellCheckingInspection
FORMAT = '%(asctime)s %(message)s'
logging.basicConfig(format=FORMAT)
formatter = logging.Formatter(FORMAT)
logger = logging.getLogger('fit_tool')
logger.setLevel(logging.INFO)



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class DiveSummaryMessage(DataMessage):
    ID = 268
    NAME = 'dive_summary'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=DiveSummaryMessage.NAME,
                         global_id=DiveSummaryMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             DiveSummaryReferenceMesgField(
                                 size=self.__get_field_size(definition_message, DiveSummaryReferenceMesgField.ID),
                                 growable=definition_message is None),
                             DiveSummaryReferenceIndexField(
                                 size=self.__get_field_size(definition_message, DiveSummaryReferenceIndexField.ID),
                                 growable=definition_message is None),
                             DiveSummaryAvgDepthField(
                                 size=self.__get_field_size(definition_message, DiveSummaryAvgDepthField.ID),
                                 growable=definition_message is None),
                             DiveSummaryMaxDepthField(
                                 size=self.__get_field_size(definition_message, DiveSummaryMaxDepthField.ID),
                                 growable=definition_message is None),
                             DiveSummarySurfaceIntervalField(
                                 size=self.__get_field_size(definition_message, DiveSummarySurfaceIntervalField.ID),
                                 growable=definition_message is None),
                             DiveSummaryStartCnsField(
                                 size=self.__get_field_size(definition_message, DiveSummaryStartCnsField.ID),
                                 growable=definition_message is None),
                             DiveSummaryEndCnsField(
                                 size=self.__get_field_size(definition_message, DiveSummaryEndCnsField.ID),
                                 growable=definition_message is None),
                             DiveSummaryStartN2Field(
                                 size=self.__get_field_size(definition_message, DiveSummaryStartN2Field.ID),
                                 growable=definition_message is None),
                             DiveSummaryEndN2Field(
                                 size=self.__get_field_size(definition_message, DiveSummaryEndN2Field.ID),
                                 growable=definition_message is None),
                             DiveSummaryO2ToxicityField(
                                 size=self.__get_field_size(definition_message, DiveSummaryO2ToxicityField.ID),
                                 growable=definition_message is None),
                             DiveSummaryDiveNumberField(
                                 size=self.__get_field_size(definition_message, DiveSummaryDiveNumberField.ID),
                                 growable=definition_message is None),
                             DiveSummaryBottomTimeField(
                                 size=self.__get_field_size(definition_message, DiveSummaryBottomTimeField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def reference_mesg(self) -> Optional[int]:
        field = self.get_field(DiveSummaryReferenceMesgField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @reference_mesg.setter
    def reference_mesg(self, value: int):
        field = self.get_field(DiveSummaryReferenceMesgField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def reference_index(self) -> Optional[int]:
        field = self.get_field(DiveSummaryReferenceIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @reference_index.setter
    def reference_index(self, value: int):
        field = self.get_field(DiveSummaryReferenceIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def avg_depth(self) -> Optional[float]:
        field = self.get_field(DiveSummaryAvgDepthField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @avg_depth.setter
    def avg_depth(self, value: float):
        field = self.get_field(DiveSummaryAvgDepthField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def max_depth(self) -> Optional[float]:
        field = self.get_field(DiveSummaryMaxDepthField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @max_depth.setter
    def max_depth(self, value: float):
        field = self.get_field(DiveSummaryMaxDepthField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def surface_interval(self) -> Optional[int]:
        field = self.get_field(DiveSummarySurfaceIntervalField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @surface_interval.setter
    def surface_interval(self, value: int):
        field = self.get_field(DiveSummarySurfaceIntervalField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def start_cns(self) -> Optional[int]:
        field = self.get_field(DiveSummaryStartCnsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @start_cns.setter
    def start_cns(self, value: int):
        field = self.get_field(DiveSummaryStartCnsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def end_cns(self) -> Optional[int]:
        field = self.get_field(DiveSummaryEndCnsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @end_cns.setter
    def end_cns(self, value: int):
        field = self.get_field(DiveSummaryEndCnsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def start_n2(self) -> Optional[int]:
        field = self.get_field(DiveSummaryStartN2Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @start_n2.setter
    def start_n2(self, value: int):
        field = self.get_field(DiveSummaryStartN2Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def end_n2(self) -> Optional[int]:
        field = self.get_field(DiveSummaryEndN2Field.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @end_n2.setter
    def end_n2(self, value: int):
        field = self.get_field(DiveSummaryEndN2Field.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def o2_toxicity(self) -> Optional[int]:
        field = self.get_field(DiveSummaryO2ToxicityField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @o2_toxicity.setter
    def o2_toxicity(self, value: int):
        field = self.get_field(DiveSummaryO2ToxicityField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def dive_number(self) -> Optional[int]:
        field = self.get_field(DiveSummaryDiveNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @dive_number.setter
    def dive_number(self, value: int):
        field = self.get_field(DiveSummaryDiveNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def bottom_time(self) -> Optional[float]:
        field = self.get_field(DiveSummaryBottomTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @bottom_time.setter
    def bottom_time(self, value: float):
        field = self.get_field(DiveSummaryBottomTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryReferenceMesgField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='reference_mesg',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryReferenceIndexField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='reference_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryAvgDepthField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='avg_depth',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryMaxDepthField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='max_depth',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummarySurfaceIntervalField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='surface_interval',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryStartCnsField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='start_cns',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryEndCnsField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='end_cns',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryStartN2Field(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='start_n2',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryEndN2Field(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='end_n2',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='percent',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryO2ToxicityField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='o2_toxicity',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            units='OTUs',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryDiveNumberField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='dive_number',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class DiveSummaryBottomTimeField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='bottom_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )



CRC_TABLE = (
    0x0000, 0xCC01, 0xD801, 0x1400, 0xF001, 0x3C00, 0x2800, 0xE401,
    0xA001, 0x6C00, 0x7800, 0xB401, 0x5000, 0x9C01, 0x8801, 0x4400,
)


def crc16(buffer, crc=0):
    if not buffer:
        return crc

    for byte in buffer:
        # byte_char = ord(byte)
        byte_char = byte
        # Taken verbatim from FIT SDK docs
        tmp = CRC_TABLE[crc & 0xF]
        crc = (crc >> 4) & 0x0FFF
        crc = crc ^ tmp ^ CRC_TABLE[byte_char & 0xF]
        # now compute checksum of upper four bits of byte
        tmp = CRC_TABLE[crc & 0xF]
        crc = (crc >> 4) & 0x0FFF
        crc = crc ^ tmp ^ CRC_TABLE[(byte_char >> 4) & 0xF]
    return crc



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *
from fit_tool.sub_field import SubField


class TrainingFileMessage(DataMessage):
    ID = 72
    NAME = 'training_file'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=TrainingFileMessage.NAME,
                         global_id=TrainingFileMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             TrainingFileTypeField(
                                 size=self.__get_field_size(definition_message, TrainingFileTypeField.ID),
                                 growable=definition_message is None),
                             TrainingFileManufacturerField(
                                 size=self.__get_field_size(definition_message, TrainingFileManufacturerField.ID),
                                 growable=definition_message is None),
                             TrainingFileProductField(
                                 size=self.__get_field_size(definition_message, TrainingFileProductField.ID),
                                 growable=definition_message is None),
                             TrainingFileSerialNumberField(
                                 size=self.__get_field_size(definition_message, TrainingFileSerialNumberField.ID),
                                 growable=definition_message is None),
                             TrainingFileTimeCreatedField(
                                 size=self.__get_field_size(definition_message, TrainingFileTimeCreatedField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def type(self) -> Optional[FileType]:
        field = self.get_field(TrainingFileTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @type.setter
    def type(self, value: FileType):
        field = self.get_field(TrainingFileTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def manufacturer(self) -> Optional[int]:
        field = self.get_field(TrainingFileManufacturerField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @manufacturer.setter
    def manufacturer(self, value: int):
        field = self.get_field(TrainingFileManufacturerField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def product(self) -> Optional[int]:
        field = self.get_field(TrainingFileProductField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @product.setter
    def product(self, value: int):
        field = self.get_field(TrainingFileProductField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def favero_product(self) -> Optional[int]:
        field = self.get_field(TrainingFileProductField.ID)
        type_field = self.get_field(TrainingFileManufacturerField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [263]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @favero_product.setter
    def favero_product(self, value: int):
        field = self.get_field(TrainingFileProductField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def garmin_product(self) -> Optional[int]:
        field = self.get_field(TrainingFileProductField.ID)
        type_field = self.get_field(TrainingFileManufacturerField.ID)

        is_sub_field_valid = type_field and type_field.get_value() in [1, 15, 13, 89]
        if field and field.is_valid() and is_sub_field_valid:
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @garmin_product.setter
    def garmin_product(self, value: int):
        field = self.get_field(TrainingFileProductField.ID)
        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def serial_number(self) -> Optional[int]:
        field = self.get_field(TrainingFileSerialNumberField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @serial_number.setter
    def serial_number(self, value: int):
        field = self.get_field(TrainingFileSerialNumberField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def time_created(self) -> Optional[int]:
        field = self.get_field(TrainingFileTimeCreatedField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @time_created.setter
    def time_created(self, value: int):
        field = self.get_field(TrainingFileTimeCreatedField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class TrainingFileTypeField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='type',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TrainingFileManufacturerField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='manufacturer',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TrainingFileProductField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='product',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
                SubField(
                    name='favero_product',
                    base_type=BaseType.UINT16,
                    scale=1,
                    offset=0,
                    reference_map={
                        TrainingFileManufacturerField.ID: [263]
                    }),
                SubField(
                    name='garmin_product',
                    base_type=BaseType.UINT16,
                    scale=1,
                    offset=0,
                    reference_map={
                        TrainingFileManufacturerField.ID: [1, 15, 13, 89]
                    })
            ]
        )


class TrainingFileSerialNumberField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='serial_number',
            field_id=self.ID,
            base_type=BaseType.UINT32Z,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TrainingFileTimeCreatedField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='time_created',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )



from enum import Enum, unique
from typing import Optional


@unique
class BaseType(Enum):
    ENUM = 0
    SINT8 = 1
    UINT8 = 2
    SINT16 = 131
    UINT16 = 132
    SINT32 = 133
    UINT32 = 134
    STRING = 7
    FLOAT32 = 136
    FLOAT64 = 137
    UINT8Z = 10
    UINT16Z = 139
    UINT32Z = 140
    BYTE = 13
    SINT64 = 142
    UINT64 = 143
    UINT64Z = 144

    @property
    def size(self) -> int:
        if self == BaseType.ENUM:
            return 1
        elif self == BaseType.SINT8:
            return 1
        elif self == BaseType.UINT8:
            return 1
        elif self == BaseType.SINT16:
            return 2
        elif self == BaseType.UINT16:
            return 2
        elif self == BaseType.SINT32:
            return 4
        elif self == BaseType.UINT32:
            return 4
        elif self == BaseType.STRING:
            return 1
        elif self == BaseType.FLOAT32:
            return 4
        elif self == BaseType.FLOAT64:
            return 8
        elif self == BaseType.UINT8Z:
            return 1
        elif self == BaseType.UINT16Z:
            return 2
        elif self == BaseType.UINT32Z:
            return 4
        elif self == BaseType.BYTE:
            return 1
        elif self == BaseType.SINT64:
            return 8
        elif self == BaseType.UINT64:
            return 8
        elif self == BaseType.UINT64Z:
            return 8
        else:
            return 0

    def is_integer(self) -> bool:
        if self in [BaseType.SINT8, BaseType.UINT8, BaseType.SINT16, BaseType.UINT16, BaseType.UINT16Z, BaseType.SINT32,
                    BaseType.UINT32, BaseType.UINT32Z, BaseType.SINT64, BaseType.UINT64, BaseType.UINT64Z]:
            return True
        else:
            return False

    def is_signed_integer(self) -> bool:
        if self in [BaseType.SINT8, BaseType.SINT16, BaseType.SINT32, BaseType.SINT64]:
            return True
        else:
            return False

    def is_big(self) -> bool:
        if self in [BaseType.SINT64, BaseType.UINT64, BaseType.UINT64Z]:
            return True
        else:
            return False

    def is_string(self) -> bool:
        return self == BaseType.STRING

    def is_float(self) -> bool:
        if self in [BaseType.FLOAT32, BaseType.FLOAT64]:
            return True
        else:
            return False

    def is_valid(self, value) -> bool:
        if value is None:
            return False

        if self.min is None or self.max is None:
            return True

        return self.min <= value <= self.max

    def invalid_raw_value(self) -> int:
        if self == BaseType.ENUM:
            return 0xff
        elif self == BaseType.SINT8:
            return 0x7f
        elif self == BaseType.UINT8:
            return 0xff
        elif self == BaseType.SINT16:
            return 0x7fff
        elif self == BaseType.UINT16:
            return 0xffff
        elif self == BaseType.SINT32:
            return 0x7fffffff
        elif self == BaseType.UINT32:
            return 0xffffffff
        elif self == BaseType.STRING:
            return 0x00
        elif self == BaseType.FLOAT32:
            return 0xffffffff
        elif self == BaseType.FLOAT64:
            return 0xffffffffffffffff
        elif self == BaseType.UINT8Z:
            return 0x00
        elif self == BaseType.UINT16Z:
            return 0x0000
        elif self == BaseType.UINT32Z:
            return 0x00000000
        elif self == BaseType.BYTE:
            return 0xff
        elif self == BaseType.SINT64:
            return 0x7fffffffffffffff
        elif self == BaseType.UINT64:
            return 0xffffffffffffffff
        elif self == BaseType.UINT64Z:
            return 0x0000000000000000
        else:
            return 0

    @property
    def max(self) -> Optional[int]:
        if self == BaseType.ENUM:
            return 0xff
        elif self == BaseType.SINT8:
            return 0x7f
        elif self == BaseType.UINT8:
            return 0xff
        elif self == BaseType.SINT16:
            return 0x7fff
        elif self == BaseType.UINT16:
            return 0xffff
        elif self == BaseType.SINT32:
            return 0x7fffffff
        elif self == BaseType.UINT32:
            return 0xffffffff
        elif self == BaseType.STRING:
            return None
        elif self == BaseType.FLOAT32:
            return None
        elif self == BaseType.FLOAT64:
            return None
        elif self == BaseType.UINT8Z:
            return 0xff
        elif self == BaseType.UINT16Z:
            return 0xffff
        elif self == BaseType.UINT32Z:
            return 0xffffffff
        elif self == BaseType.BYTE:
            return 0xff
        elif self == BaseType.SINT64:
            return 0x7fffffffffffffff
        elif self == BaseType.UINT64:
            return 0xffffffffffffffff
        elif self == BaseType.UINT64Z:
            return 0xffffffffffffffff
        else:
            return 0

    @property
    def min(self) -> Optional[int]:
        if self == BaseType.ENUM:
            return 0x00
        elif self == BaseType.SINT8:
            return -0x80
        elif self == BaseType.UINT8:
            return 0x00
        elif self == BaseType.SINT16:
            return -0x8000
        elif self == BaseType.UINT16:
            return 0x0000
        elif self == BaseType.SINT32:
            return -0x80000000
        elif self == BaseType.UINT32:
            return 0x00000000
        elif self == BaseType.STRING:
            return None
        elif self == BaseType.FLOAT32:
            return None
        elif self == BaseType.FLOAT64:
            return None
        elif self == BaseType.UINT8Z:
            return 0x00
        elif self == BaseType.UINT16Z:
            return 0x0000
        elif self == BaseType.UINT32Z:
            return 0x00000000
        elif self == BaseType.BYTE:
            return 0x00
        elif self == BaseType.SINT64:
            return -0x8000000000000000
        elif self == BaseType.UINT64:
            return 0x0000000000000000
        elif self == BaseType.UINT64Z:
            return 0x0000000000000000
        else:
            return None

    @classmethod
    def from_name(cls, name: str):
        if name == 'enum':
            return cls.ENUM
        elif name == 'sint8':
            return cls.SINT8
        elif name == 'uint8':
            return BaseType.UINT8
        elif name == 'sint16':
            return BaseType.SINT16
        elif name == 'uint16':
            return BaseType.UINT16
        elif name == 'sint32':
            return BaseType.SINT32
        elif name == 'uint32':
            return BaseType.UINT32
        elif name == 'string':
            return BaseType.STRING
        elif name == 'float32':
            return BaseType.FLOAT32
        elif name == 'float64':
            return BaseType.FLOAT64
        elif name == 'uint8z':
            return BaseType.UINT8Z
        elif name == 'uint16z':
            return BaseType.UINT16Z
        elif name == 'uint32z':
            return BaseType.UINT32Z
        elif name == 'byte':
            return BaseType.BYTE
        elif name == 'sint64':
            return BaseType.SINT64
        elif name == 'uint64':
            return BaseType.UINT64
        elif name == 'uint64z':
            return BaseType.UINT64Z
        else:
            return None


class FieldType:

    def __init__(self, name: str, base_type: BaseType):
        self.name = name
        self.base_type = base_type
        self.names_by_value = {}
        self.values_by_name = {}

    def add_value(self, name, value):
        self.names_by_value[value] = name
        self.values_by_name[name] = value

    def get_value_by_name(self, name):
        return self.values_by_name.get(name)

    def get_name_by_value(self, value):
        return self.names_by_value.get(value)



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field


class SetMessage(DataMessage):
    ID = 225
    NAME = 'set'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=SetMessage.NAME,
                         global_id=SetMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             SetDurationField(
                                 size=self.__get_field_size(definition_message, SetDurationField.ID),
                                 growable=definition_message is None),
                             SetRepetitionsField(
                                 size=self.__get_field_size(definition_message, SetRepetitionsField.ID),
                                 growable=definition_message is None),
                             SetWeightField(
                                 size=self.__get_field_size(definition_message, SetWeightField.ID),
                                 growable=definition_message is None),
                             SetSetTypeField(
                                 size=self.__get_field_size(definition_message, SetSetTypeField.ID),
                                 growable=definition_message is None),
                             SetStartTimeField(
                                 size=self.__get_field_size(definition_message, SetStartTimeField.ID),
                                 growable=definition_message is None),
                             SetCategoryField(
                                 size=self.__get_field_size(definition_message, SetCategoryField.ID),
                                 growable=definition_message is None),
                             SetCategorySubtypeField(
                                 size=self.__get_field_size(definition_message, SetCategorySubtypeField.ID),
                                 growable=definition_message is None),
                             SetWeightDisplayUnitField(
                                 size=self.__get_field_size(definition_message, SetWeightDisplayUnitField.ID),
                                 growable=definition_message is None),
                             SetMessageIndexField(
                                 size=self.__get_field_size(definition_message, SetMessageIndexField.ID),
                                 growable=definition_message is None),
                             SetWorkoutStepIndexField(
                                 size=self.__get_field_size(definition_message, SetWorkoutStepIndexField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def duration(self) -> Optional[float]:
        field = self.get_field(SetDurationField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @duration.setter
    def duration(self, value: float):
        field = self.get_field(SetDurationField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def repetitions(self) -> Optional[int]:
        field = self.get_field(SetRepetitionsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @repetitions.setter
    def repetitions(self, value: int):
        field = self.get_field(SetRepetitionsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def weight(self) -> Optional[float]:
        field = self.get_field(SetWeightField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @weight.setter
    def weight(self, value: float):
        field = self.get_field(SetWeightField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def set_type(self) -> Optional[int]:
        field = self.get_field(SetSetTypeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @set_type.setter
    def set_type(self, value: int):
        field = self.get_field(SetSetTypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def start_time(self) -> Optional[int]:
        field = self.get_field(SetStartTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @start_time.setter
    def start_time(self, value: int):
        field = self.get_field(SetStartTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def category(self) -> Optional[list[int]]:
        field = self.get_field(SetCategoryField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @category.setter
    def category(self, value: list[int]):
        field = self.get_field(SetCategoryField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def category_subtype(self) -> Optional[list[int]]:
        field = self.get_field(SetCategorySubtypeField.ID)
        if field and field.is_valid():
            return field.get_values()
        else:
            return None

    @category_subtype.setter
    def category_subtype(self, value: list[int]):
        field = self.get_field(SetCategorySubtypeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                field.set_values(value)

    @property
    def weight_display_unit(self) -> Optional[int]:
        field = self.get_field(SetWeightDisplayUnitField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @weight_display_unit.setter
    def weight_display_unit(self, value: int):
        field = self.get_field(SetWeightDisplayUnitField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(SetMessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(SetMessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def workout_step_index(self) -> Optional[int]:
        field = self.get_field(SetWorkoutStepIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @workout_step_index.setter
    def workout_step_index(self, value: int):
        field = self.get_field(SetWorkoutStepIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class TimestampField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class SetDurationField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='duration',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1000,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class SetRepetitionsField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='repetitions',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetWeightField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='weight',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=16,
            size=size,
            units='kg',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class SetSetTypeField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='set_type',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetStartTimeField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='start_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class SetCategoryField(Field):
    ID = 7

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='category',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetCategorySubtypeField(Field):
    ID = 8

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='category_subtype',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetWeightDisplayUnitField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='weight_display_unit',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetMessageIndexField(Field):
    ID = 10

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class SetWorkoutStepIndexField(Field):
    ID = 11

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='wkt_step_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



# Autogenerated. Do not modify.
#
# Profile: 21.60
from typing import List as list
from typing import Optional

from fit_tool.base_type import BaseType
from fit_tool.data_message import DataMessage
from fit_tool.definition_message import DefinitionMessage
from fit_tool.developer_field import DeveloperField
from fit_tool.endian import Endian
from fit_tool.field import Field
from fit_tool.profile.profile_type import *


class TotalsMessage(DataMessage):
    ID = 33
    NAME = 'totals'

    @staticmethod
    def __get_field_size(definition_message: DefinitionMessage, field_id: int) -> int:
        size = 0
        if definition_message:
            field_definition = definition_message.get_field_definition(field_id)
            if field_definition:
                size = field_definition.size

        return size

    def __init__(self, definition_message=None, developer_fields=None, local_id: int = 0,
                 endian: Endian = Endian.LITTLE):
        super().__init__(name=TotalsMessage.NAME,
                         global_id=TotalsMessage.ID,
                         local_id=definition_message.local_id if definition_message else local_id,
                         endian=definition_message.endian if definition_message else endian,
                         definition_message=definition_message,
                         developer_fields=developer_fields,
                         fields=[
                             MessageIndexField(
                                 size=self.__get_field_size(definition_message, MessageIndexField.ID),
                                 growable=definition_message is None),
                             TimestampField(
                                 size=self.__get_field_size(definition_message, TimestampField.ID),
                                 growable=definition_message is None),
                             TotalsTimerTimeField(
                                 size=self.__get_field_size(definition_message, TotalsTimerTimeField.ID),
                                 growable=definition_message is None),
                             TotalsDistanceField(
                                 size=self.__get_field_size(definition_message, TotalsDistanceField.ID),
                                 growable=definition_message is None),
                             TotalsCaloriesField(
                                 size=self.__get_field_size(definition_message, TotalsCaloriesField.ID),
                                 growable=definition_message is None),
                             TotalsSportField(
                                 size=self.__get_field_size(definition_message, TotalsSportField.ID),
                                 growable=definition_message is None),
                             TotalsElapsedTimeField(
                                 size=self.__get_field_size(definition_message, TotalsElapsedTimeField.ID),
                                 growable=definition_message is None),
                             TotalsSessionsField(
                                 size=self.__get_field_size(definition_message, TotalsSessionsField.ID),
                                 growable=definition_message is None),
                             TotalsActiveTimeField(
                                 size=self.__get_field_size(definition_message, TotalsActiveTimeField.ID),
                                 growable=definition_message is None),
                             TotalsSportIndexField(
                                 size=self.__get_field_size(definition_message, TotalsSportIndexField.ID),
                                 growable=definition_message is None)
                         ])

        self.growable = self.definition_message is None

    @classmethod
    def from_bytes(cls, definition_message: DefinitionMessage, developer_fields: list[DeveloperField],
                   bytes_buffer: bytes, offset: int = 0):
        message = cls(definition_message=definition_message, developer_fields=developer_fields)
        message.read_from_bytes(bytes_buffer, offset)
        return message

    @property
    def message_index(self) -> Optional[int]:
        field = self.get_field(MessageIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @message_index.setter
    def message_index(self, value: int):
        field = self.get_field(MessageIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @property
    def timestamp(self) -> Optional[int]:
        field = self.get_field(TimestampField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    # timestamp : milliseconds from January 1st, 1970 at 00:00:00 UTC

    @timestamp.setter
    def timestamp(self, value: int):
        field = self.get_field(TimestampField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def timer_time(self) -> Optional[int]:
        field = self.get_field(TotalsTimerTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @timer_time.setter
    def timer_time(self, value: int):
        field = self.get_field(TotalsTimerTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def distance(self) -> Optional[int]:
        field = self.get_field(TotalsDistanceField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @distance.setter
    def distance(self, value: int):
        field = self.get_field(TotalsDistanceField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def calories(self) -> Optional[int]:
        field = self.get_field(TotalsCaloriesField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @calories.setter
    def calories(self, value: int):
        field = self.get_field(TotalsCaloriesField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sport(self) -> Optional[Sport]:
        field = self.get_field(TotalsSportField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @sport.setter
    def sport(self, value: Sport):
        field = self.get_field(TotalsSportField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def elapsed_time(self) -> Optional[int]:
        field = self.get_field(TotalsElapsedTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @elapsed_time.setter
    def elapsed_time(self, value: int):
        field = self.get_field(TotalsElapsedTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sessions(self) -> Optional[int]:
        field = self.get_field(TotalsSessionsField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @sessions.setter
    def sessions(self, value: int):
        field = self.get_field(TotalsSessionsField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def active_time(self) -> Optional[int]:
        field = self.get_field(TotalsActiveTimeField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @active_time.setter
    def active_time(self, value: int):
        field = self.get_field(TotalsActiveTimeField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)

    @property
    def sport_index(self) -> Optional[int]:
        field = self.get_field(TotalsSportIndexField.ID)
        if field and field.is_valid():
            sub_field = field.get_valid_sub_field(self.fields)
            return field.get_value(sub_field=sub_field)
        else:
            return None

    @sport_index.setter
    def sport_index(self, value: int):
        field = self.get_field(TotalsSportIndexField.ID)

        if field:
            if value is None:
                field.clear()
            else:
                sub_field = field.get_valid_sub_field(self.fields)
                field.set_value(0, value, sub_field)


class MessageIndexField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='message_index',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timestamp',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=-631065600000,
            scale=0.001,
            size=size,
            units='ms',
            type_name='date_time',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsTimerTimeField(Field):
    ID = 0

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='timer_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsDistanceField(Field):
    ID = 1

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='distance',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='m',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsCaloriesField(Field):
    ID = 2

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='calories',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='kcal',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsSportField(Field):
    ID = 3

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sport',
            field_id=self.ID,
            base_type=BaseType.ENUM,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsElapsedTimeField(Field):
    ID = 4

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='elapsed_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsSessionsField(Field):
    ID = 5

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sessions',
            field_id=self.ID,
            base_type=BaseType.UINT16,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsActiveTimeField(Field):
    ID = 6

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='active_time',
            field_id=self.ID,
            base_type=BaseType.UINT32,
            offset=0,
            scale=1,
            size=size,
            units='s',
            type_name='',
            growable=growable,
            sub_fields=[
            ]
        )


class TotalsSportIndexField(Field):
    ID = 9

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(
            name='sport_index',
            field_id=self.ID,
            base_type=BaseType.UINT8,
            offset=0,
            scale=1,
            size=size,
            growable=growable,
            sub_fields=[
            ]
        )



from setuptools import setup

setup(name='fit-tool',
      version='0.9.14',
      description='A library for reading and writing Garmin FIT files.',
      long_description=open('README.md', 'r').read(),
      long_description_content_type='text/markdown',
      url='https://bitbucket.org/stagescycling/python_fit_tool.git',
      author='Matt Tucker',
      author_email='mtucker@stagescycling.com',
      license='none',
      packages=['fit_tool', 'fit_tool.profile', 'fit_tool.utils', 'fit_tool.profile.messages'],
      install_requires=['openpyxl==2.5.12', 'bitstruct==8.11.1'],
      zip_safe=False,
      test_suite='nose.collector',
      tests_require=['nose'],
      scripts=['bin/fittool'])
